// 写一个 TCP 并发的服务器，实现 echo 功能

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

int main(int argc, char const *argv[])
{
    int sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0)
    {
        perror("socket");
        return -1;
    }

    struct sockaddr_in server_addr;
    memset(&server_addr, 0, sizeof(server_addr));
    server_addr.sin_family = AF_INET;                // ipv4
    server_addr.sin_port = htons(8000);              // port
    server_addr.sin_addr.s_addr = htonl(INADDR_ANY); // 服务端的ip地址 : 默认接收任意ip地址的连接

    if (bind(sockfd, (struct sockaddr *)&server_addr, sizeof(server_addr)) != 0)
    {
        perror("bind");
        close(sockfd); // 关闭 socket 套接字描述符
        return -1;
    }

    listen(sockfd, 5); // 创建队列长度为5的监听队列

    while (1)
    {
        struct sockaddr_in client_addr; // 客户端地址信息
        memset(&client_addr, 0, sizeof(client_addr));
        socklen_t client_addr_len = sizeof(client_addr); // 地址信息长度

        int client_fd = accept(sockfd, (struct sockaddr *)&client_addr, &client_addr_len); // 从监听队列中取出一个连接
        if (client_fd < 0)
        {
            perror("accept");
            continue;
        }

        char client_ip[INET_ADDRSTRLEN] = "";                                    // 用于保存客户端的ip地址
        inet_ntop(AF_INET, &client_addr.sin_addr, client_ip, sizeof(client_ip)); // 将网络字节序的ip地址转换为点分十进制的ip地址
        printf("客户端 %s 连接成功\n", client_ip);

        pid_t pid = fork(); // 创建子进程
        if (pid == 0)
        {
            close(sockfd); // 关闭子进程中的监听套接字描述符

            while (1)
            {
                char buf[128] = "";
                ssize_t recv_len = recv(client_fd, buf, sizeof(buf), 0); // 接收客户端发送的数据
                if (recv_len > 0)
                {
                    char send_buf[1024] = "";
                    sprintf(send_buf, "收到 %s 发送的数据: %s", client_ip, buf);

                    send(client_fd, send_buf, strlen(send_buf), 0); // 发送数据给客户端

                    if (strncmp(buf, "exit", 4) == 0)
                    {
                        printf("客户端 %s 断开连接\n", client_ip);
                        break;
                    }
                }
            }

            close(client_fd); // 关闭客户端连接套接字描述符
            printf("客户端 %s 断开连接\n", client_ip);
            exit(0); // 子进程退出
        }
    }

    close(sockfd); // 关闭监听套接字描述符

    return 0;
}
